/****************************************************************************
*  Copyright (C) 2012 by Michael Fischer
*
*  All rights reserved.
*
*  Redistribution and use in source and binary forms, with or without 
*  modification, are permitted provided that the following conditions 
*  are met:
*  
*  1. Redistributions of source code must retain the above copyright 
*     notice, this list of conditions and the following disclaimer.
*  2. Redistributions in binary form must reproduce the above copyright
*     notice, this list of conditions and the following disclaimer in the 
*     documentation and/or other materials provided with the distribution.
*  3. Neither the name of the author nor the names of its contributors may 
*     be used to endorse or promote products derived from this software 
*     without specific prior written permission.
*
*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
*  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
*  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
*  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL 
*  THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
*  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
*  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS 
*  OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
*  AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, 
*  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF 
*  THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF 
*  SUCH DAMAGE.
*
****************************************************************************/

#include <arch/nios2.h>
#include <dev/irqreg.h>

/* Nios II */
#include "system.h"
#include "arch/nios2/nios_io.h"
#include "altera_avalon_pio_regs.h"


#define IRQ_ID 1

static int Interrupt1Ctl(int cmd, void *param);

IRQ_HANDLER sig_INTERRUPT1 = {
#ifdef NUT_PERFMON
    0,                  /* Interrupt counter, ir_count. */
#endif
    NULL,               /* Passed argument, ir_arg. */
    NULL,               /* Handler subroutine, ir_handler. */
    Interrupt1Ctl       /* Interrupt control, ir_ctl. */
};

/*!
 * \brief External interrupt 1 entry.
 */
void Interrupt1Entry(void *context)
{
   IRQ_ENTRY();

   (void)context;
   
#if (1 == PIO_INT1_BIT_CLEARING_EDGE_REGISTER)
   IOWR_ALTERA_AVALON_PIO_EDGE_CAP(PIO_INT1_BASE, 1);
#endif   
   
#ifdef NUT_PERFMON
   sig_INTERRUPT1.ir_count++;
#endif
   if (sig_INTERRUPT1.ir_handler) 
   {
      (sig_INTERRUPT1.ir_handler) (sig_INTERRUPT1.ir_arg);
   }
   IRQ_EXIT();
}

/*!
 * \brief External interrupt 1 control.
 *
 * \param cmd   Control command.
 *              - NUT_IRQCTL_INIT Initialize and disable interrupt.
 *              - NUT_IRQCTL_STATUS Query interrupt status.
 *              - NUT_IRQCTL_ENABLE Enable interrupt.
 *              - NUT_IRQCTL_DISABLE Disable interrupt.
 *              - NUT_IRQCTL_GETPRIO Query interrupt priority.
 *              - NUT_IRQCTL_SETPRIO Set interrupt priority.
 *              - NUT_IRQCTL_GETCOUNT Query and clear interrupt counter.
 * \param param Pointer to optional parameter.
 *
 * \return 0 on success, -1 otherwise.
 */
static int Interrupt1Ctl(int cmd, void *param)
{
   int rc = 0;
   unsigned int *ival = (unsigned int *)param;
   int enabled = alt_ic_irq_enabled(0, IRQ_ID);
   
   /*
    * Disable interrupt first if it is enabled.
    */
   if (enabled) 
   {
      alt_ic_irq_disable(0, IRQ_ID);
   }

   switch(cmd) 
   {
      case NUT_IRQCTL_INIT:
#if defined(PIO_INT1_BASE)
         /* Disable HW-IRQ first */
         IOWR_ALTERA_AVALON_PIO_IRQ_MASK(PIO_INT1_BASE, 0);
#endif
#if (1 == PIO_INT1_BIT_CLEARING_EDGE_REGISTER)
         IOWR_ALTERA_AVALON_PIO_EDGE_CAP(PIO_INT1_BASE, 1);
#endif   
         rc = alt_ic_isr_register(0, IRQ_ID, Interrupt1Entry, NULL, NULL);
         alt_ic_irq_disable(0, IRQ_ID);   /* Disable interrupt */
         break;
      case NUT_IRQCTL_STATUS:
         if (enabled)
         {
            *ival |= 1;
         }
         else 
         {
            *ival &= ~1;
         }
         break;
      case NUT_IRQCTL_ENABLE:
         enabled = 1;
         break;
      case NUT_IRQCTL_DISABLE:
         enabled = 0;
         break;
      case NUT_IRQCTL_GETMODE:
         *ival = NUT_IRQMODE_HIGHLEVEL;
         break;
      case NUT_IRQCTL_SETMODE:
         break;
      case NUT_IRQCTL_GETPRIO:
         *ival = IRQ_ID;
         break;
      case NUT_IRQCTL_SETPRIO:
         break;
#ifdef NUT_PERFMON
      case NUT_IRQCTL_GETCOUNT:
         *ival = (unsigned int)sig_INTERRUPT1.ir_count;
         sig_INTERRUPT1.ir_count = 0;
         break;
#endif
      default:
         rc = -1;
         break;
   }
   
   /* Enable interrupt. */
   if (enabled) 
   {
      alt_ic_irq_enable(0, IRQ_ID);
#if defined(PIO_INT1_BASE)
      IOWR_ALTERA_AVALON_PIO_IRQ_MASK(PIO_INT1_BASE, 1);
#endif   
   }
   else
   {
      alt_ic_irq_disable(0, IRQ_ID);
#if defined(PIO_INT1_BASE)
      IOWR_ALTERA_AVALON_PIO_IRQ_MASK(PIO_INT1_BASE, 0);
#endif   
   }

   return(rc);
}

/*** EOF ***/


